import os, sys, time


class RunCommandError (Exception):
	def __init__ (self, value):
		self.value = value
	
	def __str__ (self):
		return self.value


class ActionLog:
	"Log and Notice actions."

	__wheelType	= ["-", "\\", "|", "/"]
	__wheelStop	= 0

	def __init__ (self, log = "&1"):

		self.__logFile		= ""
		self.__logFd		= None
		self.__noticeLevel	= 1

		if log is None:
			self.__logFile	= ""
		else:
			if log[0] == "&":
				self.__logFile	= log
			elif log[0] == "/":
				self.__logFile	= log
				self.__logFd = open (self.__logFile, "a")
			else:
				self.__logFile	= os.getcwd () + "/" + log
				self.__logFd = open (self.__logFile, "a")



	def WriteLogHead (self, str):
		self.__genTime		= time.strftime("%a, %d %b %Y %H:%M:%S %Z\n", 
							 time.localtime(time.time ()))
		if self.__logFd is not None:
			self.__logFd.seek (0)
			self.__logFd.truncate ()
			line = ""
			for i in range (1, len (str + self.__genTime) + 4):
				line = line + "="
			line = line + "\n\n"

			self.__logFd.write (str + " -- " + self.__genTime + line)


	def AppendLog (self, str):
		if self.__logFd is not None:
			self.__logFd.seek (0, 2)
			self.__logFd.write (str)
			self.__logFd.flush ()


	def Printf (self, str):
			self.AppendLog ("\nPRINT: " + str + "\n\n")


	def System (self, command):
		if self.__logFile == "":
			ret = os.system (command + " >>/dev/null 2>&1")
		elif self.__logFile[0] == "&":
			self.AppendLog ("\n> " + command)
			ret = os.system (command + " >" + self.__logFile)
		else:
			self.AppendLog ("\n> " + command + "\n")
			ret = os.system (command + " >>" + self.__logFile + " 2>&1")

		if ret:
			# Something wrong when running the command
			raise RunCommandError (command)

	def Popen (self, command, mode):

		maxTry = 3

		for i in range (0, maxTry):
			if self.__logFile == "":
				fd = os.popen (command + " 2>/dev/null", mode)
			elif self.__logFile[0] == "&":
				self.AppendLog ("> " + command)
				fd = os.popen (command, mode) 
			else:
				self.AppendLog ("> " + command + "\n")
				fd = os.popen (command + " 2>>" + self.__logFile, mode) 
		
			str = fd.read ()
			ret = fd.close ()
			if ret is None:
				self.AppendLog (str + "\n")
				return str

			time.sleep (1)

		raise RunCommandError (command)


	def SetNoticeLevel (self, level):
		self.__noticeLevel = level


	def Notice (self, level, note):
		if level <= self.__noticeLevel:
			sys.stdout.write (note)
			sys.stdout.flush ()


	def Error (self, str):
		sys.stderr.write ("[ Error: " + str + " ]\n")


	def Wheeling (self):
		self.__wheelStop = (self.__wheelStop + 1) % len (self.__wheelType)
		sys.stderr.write ("\010" + self.__wheelType[self.__wheelStop])
