#!/usr/bin/python

from ActionLog import RunCommandError
from SnmpGrepValue import *
from NetSnmp import *

class POSIXEvent:
	"POSIX Event log root class"

	def __init__ (self, snmp):
		self.Snmp	= snmp


class PEventLog (POSIXEvent):
	"POSIX Event Log information"

	defaultPath	= None
	defaultQuery	= None
	lastRecordId	= None
	Index		= []
	Size		= []
	Format		= []
	Type		= []
	Facility	= []
	Severity	= []
	UID		= []
	GID		= []
	PID		= []
	PGID		= []
	timeStamp	= []
	Flag		= []
	Thread		= []
	Processor	= []
	varString	= []
	varBinary	= []


	def GetLogDefaultPath (self):
		self.defaultPath	= self.Snmp.SnmpGetNext ("LELLogDefaultPath", String)


	def GetLogDefaultQuery (self):
		self.defaultQuery	= self.Snmp.SnmpGetNext ("LELLogDefaultQuery", Gauge32)


	def SetLogDefaultQuery (self, qstr):
		self.Snmp.SnmpSet ("LELLogDefaultQuery.0 = " + str (qstr))


	def GetLogLastRecordId (self):
		self.defaultQuery	= self.Snmp.SnmpGetNext ("LELLogLastRecordID", Gauge32)
		

	def GetTableIndex (self):
		qstr = "lelLogSize"
		
		while 1:
			try:
				i = self.Snmp.GetTableIndex (qstr)
			except SnmpResultCheckError:
				break
			self.Index.append (i)
			qstr = "lelLogSize." + i


	def GetAttributes (self):
		self.Snmp.al.Notice (3, "Getting EventLog Attributes  ")
		self.GetLogDefaultPath ()
		self.GetLogDefaultQuery ()
		self.GetLogLastRecordId ()
		self.GetTableIndex ()

		for i in self.Index:
			self.Size.append (self.Snmp.SnmpGet ("lelLogSize." + i, Gauge32))
			self.Format.append (self.Snmp.SnmpGet ("lelLogFormat." + i, String))
			self.Type.append (self.Snmp.SnmpGet ("lelLogType." + i, Integer))
			self.Facility.append (self.Snmp.SnmpGet ("lelLogFacility." + i, String))
			self.Severity.append (self.Snmp.SnmpGet ("lelLogSeverity." + i, String))
			self.UID.append (self.Snmp.SnmpGet ("lelLogUID." + i, Gauge32))
			self.GID.append (self.Snmp.SnmpGet ("lelLogGID." + i, Gauge32))
			self.PID.append (self.Snmp.SnmpGet ("lelLogPID." + i, Gauge32))
			self.PGID.append (self.Snmp.SnmpGet ("lelLogPGID." + i, Gauge32))
			self.timeStamp.append (self.Snmp.SnmpGet ("lelLogTimeStamp." + i, String))
			self.Flag.append (self.Snmp.SnmpGet ("lelLogFlag." + i, Gauge32))
			self.Thread.append (self.Snmp.SnmpGet ("lelLogThread." + i, Gauge32))
			self.Processor.append (self.Snmp.SnmpGet ("lelLogProcessor." + i, Gauge32))
			self.varString.append (self.Snmp.SnmpGet ("lelLogVarString." + i, String))
			self.varBinary.append (self.Snmp.SnmpGet ("lelLogVarBinary." + i, String))


	def ShowAttributes (self):
		print "Default Path =", self.defaultPath
		print "Default Query =", self.defaultQuery
		print "EventLog Table:"

		num = 0
		for i in self.Index:
			print "Item", num, ":"
			print "\tSize[" + i + "] =", self.Size[num]
			print "\tFormat[" + i + "] =", self.Format[num]
			print "\tType[" + i + "] =", self.Type[num]
			print "\tFacility[" + i + "] =", self.Facility[num]
			print "\tSeverity[" + i + "] =", self.Severity[num]
			print "\tUID[" + i + "] =", self.UID[num]
			print "\tGID[" + i + "] =", self.GID[num]
			print "\tPID[" + i + "] =", self.PID[num]
			print "\tPGID[" + i + "] =", self.PGID[num]
			print "\tTimeStamp[" + i + "] =", self.timeStamp[num]
			print "\tFlag[" + i + "] =", self.Flag[num]
			print "\tThread[" + i + "] =", self.Thread[num]
			print "\tProcessor[" + i + "] =", self.Processor[num]
			print "\tVarString[" + i + "] =", self.varString[num]
			print "\tVarBinary[" + i + "] =", self.varBinary[num]
			num = num + 1



class PEventQuery (POSIXEvent):
	"POSIX Event Query object"

	lelQueryIndex		= []
	eventQueryString	= []
	eventQueryPurpose	= []
	eventQueryRefCount	= []
	eventQueryRowStatus	= []


	def CreateEventQuery (self, qstr, purpose):
		qstr = "'" + qstr + "'"
		index = self.Snmp.SnmpGetNext ("LELQueryNextIndex", Gauge32)
		num = str (index) + " = "
		self.Snmp.SnmpSet ("lelQueryRowStatus." + num + "createAndGo" +
				   " lelQueryString." + num + qstr +
				   " lelQueryPurpose." + num + str (purpose))
		self.Snmp.SnmpSet ("lelQueryRowStatus." + num + "active")
		return index


	def DestroyEventQuery (self, index):
		if int (self.Snmp.SnmpGet ("lelQueryRefCount." + str (index), Integer)) > 0:
			return index
		self.Snmp.SnmpSet ("lelQueryRowStatus." + str (index) + " = destroy")
		return 0



class PEventNotify (POSIXEvent):
	"POSIX Event Notify object"

	Index		= []
	queryId		= []
	onceOnly	= []
	trapGeneric	= []
	rowStatus	= []

	
	def CreateEventNotify (self, query, once, trap):
		index = self.Snmp.SnmpGetNext ("LELNotifyNextIndex", Gauge32)
		num = str (index) + " = "
		self.Snmp.SnmpSet ("lelNotifyRowStatus." + num + "createAndGo" +
				   " lelNotifyQueryId." + num + str (query) +
				   " lelNotifyOnceOnly." + num + str (once) +
				   " lelNotifyTrapGeneric." + num + str (trap))
		self.Snmp.SnmpSet ("lelNotifyRowStatus." + num + "active")
		return index


	def DestroyEventNotify (self, index):
		self.Snmp.SnmpSet ("lelNotifyRowStatus." + str (index) + " = destroy")
		return 0



class PEventHandler (POSIXEvent):
	"POSIX Event Handler object"

	Index		= []
	Command		= []
	User		= []
	refCount	= []
	rowStatus	= []


	def CreateEventHandler (self, command, user):
		index = self.Snmp.SnmpGetNext ("LELHandlerNextIndex", Gauge32)
		num = str (index) + " = "
		self.Snmp.SnmpSet ("lelHandlerRowStatus." + num + "createAndGo" +
				   " lelHandlerCommand." + num + "'" + str (command) + "'" +
				   " lelHandlerUser." + num + str (user))
		self.Snmp.SnmpSet ("lelHandlerRowStatus." + num + "active")
		return index


	def DestroyEventHandler (self, index):
		if int (self.Snmp.SnmpGet ("lelHandlerRefCount." + str (index), Integer)) > 0:
			return index
		self.Snmp.SnmpSet ("lelHandlerRowStatus." + str (index) + " = destroy")
		return 0



class PEventAction (POSIXEvent):
	"POSIX Event Action object"

	Index		= []
	queryIndex	= []
	Handler		= []
	Type		= []
	rowStatus	= []


	def CreateEventAction (self, query, handler, type):
		index = self.Snmp.SnmpGetNext ("LELActionNextIndex", Gauge32)
		num = str (index) + " = "
		self.Snmp.SnmpSet ("lelActionRowStatus." + num + "createAndGo" +
				   " lelActionQueryIndex." + num + str (query) +
				   " lelActionHandler." + num + str (handler) +
				   " lelActionType." + num + str (type))
		return index


	def DestroyEventAction (self, index):
		self.Snmp.SnmpSet ("lelActionRowStatus." + str (index) + " = destroy")



class PEventConfig (POSIXEvent):
	"POSIX Event Config object"

	discardDups	= 0
	discardInterval	= 0
	discardDupCount	= 0
	screenFilter	= 0
	lookbackSize	= 0

	def GetConfig (self):
		self.discardDups	= self.Snmp.SnmpGet ("LELConfigDiscardDups.0", Value)
		self.discardInterval	= self.Snmp.SnmpGet ("LELConfigDiscardInterval.0", Gauge32)
		self.discardDupCount	= self.Snmp.SnmpGet ("LELConfigDiscardDupCount.0", Gauge32)
		self.discardFilter	= self.Snmp.SnmpGet ("LELConfigScreenFilter.0", Gauge32)
		self.lookbackSize	= self.Snmp.SnmpGet ("LELConfigLookbackSize.0", Gauge32)

	def SetConfigDiscardDups (self, dups):
		self.Snmp.SnmpSet ("LELConfigDiscardDups.0 = " + str (dups))

	def SetConfigDiscardInterval (self, interval):
		self.Snmp.SnmpSet ("LELConfigDiscardInterval.0 = " + str (interval))

	def SetConfigDiscardDupCount (self, count):
		self.Snmp.SnmpSet ("LELConfigDiscardDupCount.0 = " + str (count))

	def SetConfigScreenFilter (self, filter):
		self.Snmp.SnmpSet ("LELConfigScreenFilter.0 = " + str (filter))

	def SetConfigLookbackSize (self, size):
		self.Snmp.SnmpSet ("LELConfigLookbackSize.0 = " + str (size))



class PEventFacility (POSIXEvent):
	"POSIX Event Facility object"

	Code	= []
	Name	= []
	Kernel	= []
	Private	= []
	Filter	= []

	def GetFacility (self):
		qstr = "lelFacilityName"
                
                while 1:
                        try:    
                                i = self.Snmp.GetTableIndex (qstr)
                        except SnmpResultCheckError:
                                break
                        self.Code.append (i)
                        qstr = "lelFacilityName." + i

			self.Name.append (self.Snmp.SnmpGet ("lelFacilityName." + i, String))
			self.Kernel.append (self.Snmp.SnmpGet ("lelFacilityKernel." + i, Value))
			self.Private.append (self.Snmp.SnmpGet ("lelFacilityPrivate." + i, Value))
			self.Filter.append (self.Snmp.SnmpGet ("lelFacilityFilter." + i, String))

	def ShowFacility (self):
		num = 0
		for i in self.Code:
			print "Item", num, ":"
			print "\tName[" + i + "] =", self.Name[num]
			print "\tKernel[" + i + "] =", self.Kernel[num]
			print "\tPrivate[" + i + "] =", self.Private[num]
			print "\tFilter[" + i + "] =", self.Filter[num]
			num = num + 1
